#include "gamewidget.h"


GameWidget::GameWidget()
	:Widget(),
	time(0),
	playing(false),
	paused(false),
	justReachedTroll(false)
{
	camera.SetPlayer(&player);

	for (int i=0; i<250; i++) {
		int x = 920 + rand()%1800;
		int y = SCREEN_H - 64 + rand()%64;
		int rot = rand()%256;
		float scale = (float)(rand()%100 - 20)/80.0f + 1.0f;

		if (x < 1000) {
			y -= 1000 - x;
		}

		Stone s;
		s.x = x;
		s.y = y;
		s.rot = rot;
		s.scale = scale;
		stones.push_back(s);
	}
}



void GameWidget::Gradient(Bitmap &canvas, int x1, int y1, int x2, int y2, Color c1, Color c2) {
	float h2 = (float)(y2 - y1);

	float r1 = (float)c1.r();
	float g1 = (float)c1.g();
	float b1 = (float)c1.b();

	float r2 = (float)c2.r();
	float g2 = (float)c2.g();
	float b2 = (float)c2.b();

	float dr = (r2 - r1)/h2;
	float dg = (g2 - g1)/h2;
	float db = (b2 - b1)/h2;

	for (int i=y1; i<y2; i++) {
		hline(canvas, x1, i, x2, makecol((int)(r1+=dr), (int)(g1+=dg), (int)(b1+=db)));
	}
}


void GameWidget::Draw(Bitmap &canvas) {
	Font fnt = GetFont(0);

	// sky gradient
	Gradient(canvas, 0, 0, w()-1, h()/2, Color(100,160,255), Color(196,240,240));

	// waves
	int dx = 0;
	int dy = 0;
	int y = 0;

	y = h()/2 - 20;
	dy = (int)(6*sin(time*8.3));
	dx = (int)(20*cos(time*3.4) - camera.pos.x * 0.75) % w();
	if (dx < 0) {
		masked_blit(bmpWave1, canvas, -dx, 0, 0, y+dy, bmpWave1->w + dx, bmpWave1->h);
		masked_blit(bmpWave1, canvas, 0, 0, bmpWave1->w + dx, y+dy, -dx, bmpWave1->h);
	}
	else {
		masked_blit(bmpWave1, canvas, bmpWave1->w - dx, 0, 0, y+dy, dx, bmpWave1->h);
		masked_blit(bmpWave1, canvas, 0, 0, dx, y+dy, bmpWave1->w - dx, bmpWave1->h);
	}

	y += 16;
	dy = (int)(8*sin(time*6.1));
	dx = -(int)(30*cos(time*2.8) + camera.pos.x * 1.00) % w();
	if (dx < 0) {
		masked_blit(bmpWave2, canvas, -dx, 0, 0, y+dy, bmpWave2->w + dx, bmpWave2->h);
		masked_blit(bmpWave2, canvas, 0, 0, bmpWave2->w + dx, y+dy, -dx, bmpWave2->h);
	}
	else {
		masked_blit(bmpWave2, canvas, bmpWave2->w - dx, 0, 0, y+dy, dx, bmpWave2->h);
		masked_blit(bmpWave2, canvas, 0, 0, dx, y+dy, bmpWave2->w - dx, bmpWave2->h);
	}

	// buoys
	for (int i=0,d=10; i<5; i++, d-=2) {
		int x = (int)(i*200 - camera.pos.x + SCREEN_W/2) - bmpBuoy->w/2;
		dy = (int)(16*sin(time*4.2 + i*200));
		draw_sprite(canvas, bmpBuoy, x, y+dy-6);
		fnt.TextPrintf(canvas, Point(x+36,y+dy), Color::black, Color(-1), 2, "%d m", d);
	}

	y += 20;
	dy = (int)(12*sin(time*3.7));
	dx = (int)(60*cos(time*2.3) - camera.pos.x * 1.25) % w();
	if (dx < 0) {
		masked_blit(bmpWave3, canvas, -dx, 0, 0, y+dy, bmpWave3->w + dx, bmpWave3->h);
		masked_blit(bmpWave3, canvas, 0, 0, bmpWave3->w + dx, y+dy, -dx, bmpWave3->h);
	}
	else {
		masked_blit(bmpWave3, canvas, bmpWave3->w - dx, 0, 0, y+dy, dx, bmpWave3->h);
		masked_blit(bmpWave3, canvas, 0, 0, dx, y+dy, bmpWave3->w - dx, bmpWave3->h);
	}

	// ocean gradient
	Gradient(canvas, 0, 3*h()/4, w()-1, h(), Color(66,103,170), Color(3,166,195));

	// sun
	set_alpha_blender();
	dx = (int)(camera.pos.x * 0.2) % w();
	draw_trans_sprite(canvas, bmpSun, 5*w()/6 - 128 - dx, h()/10 - 128);

	// pier
	for (int i=0; i<9; i++) {
		int x = (int)(2600 - i*200 - camera.pos.x + SCREEN_W/2) - 24;
		draw_sprite(canvas, bmpPier, x, h()-160);
	}

	// stones
	int offx = (int)(-camera.pos.x + SCREEN_W/2 - 32);
	int offy = -32;
	for (vector<Stone>::iterator iter = stones.begin(); iter != stones.end(); ++iter) {
		rotate_scaled_sprite(canvas, bmpRock, iter->x + offx, iter->y + offy, itofix(iter->rot), ftofix(iter->scale));
	}

	// signs
	for (int i=0,d=2; i<8; i++, d+=2) {
		int x = (int)(1000 + d*100 - camera.pos.x + SCREEN_W/2) - bmpSign->w/2 - 10;
		draw_sprite(canvas, bmpSign, x, h()-210);
		fnt.TextPrintf(canvas, Point(x+36,h()-204), Color::black, Color(-1), 2, "%d m", d);
	}

	// troll
	draw_sprite(canvas, bmpTroll, (int)(2300 - camera.pos.x + SCREEN_W/2) - bmpTroll->w/2, 220);

	// player
	if (playing) {
		player.Draw(canvas, &camera);
	}

	// overlays
	if (paused) {
		textprintf_centre_ex(canvas, focusfont, w()/2+4, (h() - text_height(focusfont))/2+4, 0, -1, "Paused - (Q)uit, (C)ontinue");
		textprintf_centre_ex(canvas, focusfont, w()/2, (h() - text_height(focusfont))/2, -1, -1, "Paused - (Q)uit, (C)ontinue");
	}
	else if (justReachedTroll) {
		textprintf_centre_ex(canvas, focusfont, w()/2+4, h()/2 - 16 - 2*text_height(focusfont) + 4, 0, -1, "Congratulations!");
		textprintf_centre_ex(canvas, focusfont, w()/2, h()/2 - 16 - 2*text_height(focusfont), -1, -1, "Congratulations!");
		textprintf_centre_ex(canvas, focusfont, w()/2+4, h()/2 - 8 - text_height(focusfont) + 4, 0, -1, "You have reached the troll!");
		textprintf_centre_ex(canvas, focusfont, w()/2, h()/2 - 8 - text_height(focusfont), -1, -1, "You have reached the troll!");
		textprintf_centre_ex(canvas, focusfont, w()/2+4, h()/2 + 8 + 4, 0, -1, "A secret is now unlocked.");
		textprintf_centre_ex(canvas, focusfont, w()/2, h()/2 + 8, -1, -1, "A secret is now unlocked.");
	}
	else if (player.state == Player::TUNA_LANDED) {
		float score = player.pos.x + 85 - 1000;
		textprintf_centre_ex(canvas, focusfont, w()/2+4, (h() - text_height(focusfont))/2+4, 0, -1, "Distance: %2.2f m", score/100);
		textprintf_centre_ex(canvas, focusfont, w()/2, (h() - text_height(focusfont))/2, -1, -1, "Distance: %2.2f m", score/100);
	}
	else if (player.state == Player::TUNA_CRASHED) {
		textprintf_centre_ex(canvas, focusfont, w()/2+4, (h() - text_height(focusfont))/2+4, 0, -1, "Looser!");
		textprintf_centre_ex(canvas, focusfont, w()/2, (h() - text_height(focusfont))/2, -1, -1, "Looser!");
	}
}


void GameWidget::MsgTick() {
	Widget::MsgTick();
	time += 0.01;

	if (!paused && playing) {
		camera.Update();
		player.Update();
	}
}


void GameWidget::StartPlaying() {
	playing = true;
	player.Init();
	camera.Init();
}


void GameWidget::StopPlaying() {
	playing = false;
}


void GameWidget::Pause() {
	paused = !paused;
}


bool GameWidget::MsgChar(int c) {
	Widget::MsgChar(c);

	switch (c>>8) {
		case KEY_ESC:
		case KEY_P:
			Pause();
		break;

		case KEY_Q:
			if (paused) {
				Pause();
				StopPlaying();
				GetParent()->Close();
			}
		break;

		case KEY_C:
			if (paused) {
				Pause();
			}
		break;

		case KEY_ENTER:
		case KEY_SPACE:
			if (player.state == Player::TUNA_CRASHED || player.state == Player::TUNA_LANDED) {
				if (justReachedTroll) {
					justReachedTroll = false;
					unlockedSecret = 1;
					player.Init();
				}
				else {
					float score = player.pos.x + 85 - 1000;
					if (score >= 1250 && unlockedSecret == 0) {
						justReachedTroll = true;
					}
					else {
						player.Init();
					}
				}
			}
		break;

		default:
			return false;
	};

	return true;
}


bool GameWidget::MsgWantfocus() {
	Widget::MsgWantfocus();
	return playing;
}
