#include "global.h"

FONT *titlefont = NULL;
FONT *focusfont = NULL;
FONT *normalfont = NULL;


BITMAP *bmpWave1 = NULL;
BITMAP *bmpWave2 = NULL;
BITMAP *bmpWave3 = NULL;


BITMAP *CreateWave(int A, int freq, int col) {
	BITMAP *bmp = create_bitmap(640, A*6);
	int magenta = makecol(255,0,255);

	for (int x=0; x<640; x++) {
		int y = (int)((sin((double)x * freq * 6.28318530717958 / 640.0) + 1.0) * A / 2);
		vline(bmp, x, 0, y, magenta);
		vline(bmp, x, y, A*6-1, col);
	}

	return bmp;
}


void CreateWaves() {
	bmpWave1 = CreateWave(12, 15, makecol(100,140,140));
	bmpWave2 = CreateWave(20, 9, makecol(80,158,190));
	bmpWave3 = CreateWave(28, 6, makecol(66,103,170));
}


void DestroyWaves() {
	destroy_bitmap(bmpWave1);
	destroy_bitmap(bmpWave2);
	destroy_bitmap(bmpWave3);
}


BITMAP *bmpTuna = NULL;
BITMAP *bmpTunaShadow = NULL;
BITMAP *bmpTunaMix = NULL;
BITMAP *bmpTunaMask = NULL;
BITMAP *bmpUnderwater = NULL;


void LoadTuna(const char *filename) {
	bmpTuna = load_bitmap(filename, NULL);

	bmpTunaShadow = load_bitmap(filename, NULL);
	int magenta = makecol(255,0,255);
	for (int y=0; y<bmpTunaShadow->h; y++) {
		for (int x=0; x<bmpTunaShadow->w; x++) {
			if (getpixel(bmpTunaShadow, x, y) != magenta) {
				putpixel(bmpTunaShadow, x, y, 0);
			}
		}
	}

	bmpTunaMix = create_bitmap(bmpTuna->w/2, bmpTuna->h/2);
	bmpTunaMask = create_bitmap(bmpTuna->w/2, bmpTuna->h/2);
	clear_to_color(bmpTunaMask, magenta);
	rotate_scaled_sprite(bmpTunaMask, bmpTuna, 0, 0, 0, ftofix(0.5));
	for (int y=0; y<bmpTunaMask->h; y++) {
		for (int x=0; x<bmpTunaMask->w; x++) {
			if (getpixel(bmpTunaMask, x, y) != magenta) {
				putpixel(bmpTunaMask, x, y, makeacol(0,0,0,0));
			}
			else {
				putpixel(bmpTunaMask, x, y, makeacol(255,0,255,255));
			}
		}
	}

	bmpUnderwater = create_bitmap(bmpTuna->w/2, bmpTuna->h/2);
	clear_to_color(bmpUnderwater, makeacol(64,128,192,128));
}


void DestroyTuna() {
	destroy_bitmap(bmpTuna);
	destroy_bitmap(bmpTunaShadow);
	destroy_bitmap(bmpTunaMix);
	destroy_bitmap(bmpTunaMask);
	destroy_bitmap(bmpUnderwater);
}


BITMAP *bmpSun = NULL;
BITMAP *bmpBuoy = NULL;
BITMAP *bmpSign = NULL;
BITMAP *bmpRock = NULL;
BITMAP *bmpPier = NULL;
BITMAP *bmpTroll = NULL;

int unlockedSecret = 0;


ScrollingText::ScrollingText() : Label(), tick(0) {
}


void ScrollingText::Draw(Bitmap &canvas) {
	canvas.Clear();
	Font f = GetFont(0);
	const char *t = GetText();
	int l = f.TextLength(t);
	int dx = (tick)%l;
	f.TextPrintf(canvas, Point(-dx, 0), Color::white, -1, 0, t);

	if (-dx + l < w()) {
		f.TextPrintf(canvas, Point(-dx + l, 0), Color::white, -1, 0, t);
	}
}


void ScrollingText::MsgTick() {
	Label::MsgTick();
	++tick;
}


void ScrollingText::LoadText(const char *filename) {
	PACKFILE *f = pack_fopen(filename, "r");
	char t[4096];
	pack_fgets(t, 4096, f);
	Label::SetText(t);
	pack_fclose(f);
}
