#ifndef		__GLOBAL_H__
#define		__GLOBAL_H__


#include <math.h>
#include <MASkinG.h>
using namespace MAS;


// Interface for our game menu dialogs. All our dialogs will be derived
// from this class and will at least implement the Run() function.
class GameDialog : public Dialog {
	public:
		// This function runs the dialog and returns a unique ID of
		// the dialog that's supposed to be run next (i.e. the next
		// state of the finite state machine that is our menu system).
		virtual int Run() = 0;

};


// These are the states the menu system can be in. Every state has a
// matching dialog that is run in that state. The exception is S_EXIT_GAME
// which is a special exit state.
enum {
	S_EXIT_GAME = -1,
	S_MAIN_MENU,
	S_PLAYING,
	S_OPTIONS,
	S_HIGHSCORES,
	S_HELP,
	S_SECRETGAME,

	S_COUNT
};


extern BITMAP *bmpWave1;
extern BITMAP *bmpWave2;
extern BITMAP *bmpWave3;

void CreateWaves();
void DestroyWaves();

extern BITMAP *bmpTuna;
extern BITMAP *bmpTunaShadow;
extern BITMAP *bmpTunaMix;
extern BITMAP *bmpTunaMask;
extern BITMAP *bmpUnderwater;

void LoadTuna(const char *filename);
void DestroyTuna();

extern BITMAP *bmpSun;
extern BITMAP *bmpBuoy;
extern BITMAP *bmpSign;
extern BITMAP *bmpRock;
extern BITMAP *bmpPier;
extern BITMAP *bmpTroll;

extern FONT *titlefont;
extern FONT *focusfont;
extern FONT *normalfont;

extern int unlockedSecret;


class TunaPreview : public Button {
	protected:
		void Draw(Bitmap &canvas) {
			int offset = Selected() ? 4 : 0;
			rotate_sprite(canvas, bmpTunaShadow, -80+4+offset, 80+2+offset, itofix(64));
			rotate_sprite(canvas, bmpTuna, -80+offset, 80+offset, itofix(64));
		}
};


class CustomLabel : public Label {
	protected:
		void Draw(Bitmap &canvas) {
			textprintf_centre_ex(canvas, titlefont, w()/2+4, -18, Color::black, -1, GetText());
			textprintf_centre_ex(canvas, titlefont, w()/2, -20, -1, -1, GetText());
		}
};


class TextButton : public Button {
	protected:
		void Draw(Bitmap &canvas) {
			FONT *f = HasFocus() ? focusfont : normalfont;
			int offset = Selected() ? 2 : 0;
			textprintf_centre_ex(canvas, f, w()/2+4+offset, -8+offset, Color::black, -1, GetText());
			textprintf_centre_ex(canvas, f, w()/2+offset, -10+offset, -1, -1, GetText());
		}
};


class ScrollingText : public Label {
	protected:
		void Draw(Bitmap &canvas);
		void MsgTick();
		int tick;

	public:
		ScrollingText();
		void LoadText(const char *filename);
};


#endif		//__GLOBAL_H__
