#include "player.h"
#include "camera.h"
#include "global.h"


Player::Player() : Object() {
	min = Vector(0, 0);
	max = Vector(2400, 520);
}


Player::~Player() {
}


void Player::Init() {
	pos.x = 0.0f;
	pos.y = 450.0f;
	v.x = 0.0f;
	v.y = 0.0f;
	state = TUNA_SWIMMING;
	power = 0;
	increment = 0;
}


void Player::Update() {
	switch (state) {
		case TUNA_SWIMMING: {
			if (key[KEY_SPACE]) {
				if (power >= 100) {
					increment = -1;
				}
				else {
					if (increment != -1) {
						increment = 1;
						if (power <= 1) power = 2;
					}
				}

				power += (int)(increment*sqrt(power)*2);
			}
			else {
				if (power > 0) {
					increment = -1;
				}
				else {
					increment = 0;
				}

				power += (int)(increment*sqrt(power)*2);

				if (power == 1) power = 0;
			}

			power = MID(0,power,100);
			if (power < 0) allegro_message("yuhu");

			if (key[KEY_RIGHT]) {
				v.x += 0.4;
			}
			else if (key[KEY_LEFT]) {
				v.x -= 0.4;
			}
			else {
				v.x *= 0.92;
			}

			if (key[KEY_UP]) {
				v.y -= 0.4;
			}
			else if (key[KEY_DOWN]) {
				v.y += 0.4;
			}
			else {
				v.y *= 0.92;
			}

			v.x = MID(-16.0f, v.x, 16.0f);
			v.y = MID(-16.0f, v.y, 16.0f);

			pos.x += v.x;
			if (pos.x >= 850 || pos.x < min.x) {
				pos.x -= v.x;
				v.x = -v.x*0.5f;

				if (pos.x >= 850)
					state = TUNA_CRASHED;
			}

			pos.y += v.y;
			if (pos.y >= max.y) {
				pos.y -= v.y;
				v.y = -v.y*0.5f;
			}
			else if (pos.y < 400) {
				v.y *= 1.6;
				v.y -= (float)power * 0.6 / 5;
				v.x += (float)power * 0.25 / 5;
				state = TUNA_FLYING;
			}
		}
		break;

		case TUNA_CRASHED: {
			v.x *= 0.92;

			if (pos.y < 400) {
				// gravity
				if (v.y < 0) {
					v.y *= 0.85;
					if (fabs(v.y) < 0.75f) {
						v.y = -v.y;
					}
				}
				else {
					v.y /= 0.85;
				}
			}
			else {
				v.y *= 0.92;
			}

			v.x = MID(-16.0f, v.x, 16.0f);
			v.y = MID(-16.0f, v.y, 16.0f);

			pos.x += v.x;
			pos.y += v.y;
		}
		break;

		case TUNA_FLYING: {
			// air resistance
			//v.x *= 0.998;

			// gravity
			if (v.y < 0) {
				v.y *= 0.9;
				if (fabs(v.y) < 0.75f) {
					v.y = -v.y;
				}
			}
			else {
				v.y /= 0.9;
			}

			v.x = MID(-16.0f, v.x, 32.0f);
			v.y = MID(-32.0f, v.y, 16.0f);

			pos.x += v.x;
			pos.y += v.y;

			if (pos.y > 375 && pos.x > 850) {
				state = TUNA_LANDED;
			}
			else if (pos.y > 400 && pos.x < 850) {
				state = TUNA_SWIMMING;
			}
		}
		break;
	};
}


void Player::Draw(BITMAP *canvas, Camera *camera) {
	int x = (int)(pos.x - camera->pos.x + SCREEN_W/2) - bmpTuna->w/4;
	int y = (int)(pos.y) - bmpTuna->h/2;

	rect(canvas, 0, 471, 101, 479, makecol(0,0,0));
	if (power) {
		rectfill(canvas, 1, 472, power, 478, makecol(255,255,0));
	}

	if (pos.x > 850) {
		rotate_scaled_sprite(canvas, bmpTuna, x, y, itofix(0), ftofix(0.5));
	}
	else {
		rotate_scaled_sprite(bmpTunaMix, bmpTuna, 0, 0, itofix(0), ftofix(0.5));
		set_alpha_blender();
		draw_trans_sprite(bmpTunaMix, bmpUnderwater, 0, MAX(0, (int)(445 - pos.y)));
		draw_trans_sprite(bmpTunaMix, bmpTunaMask, 0, 0);
		draw_sprite(canvas, bmpTunaMix, x, y);
	}
}
